import XCTest
@testable import Model

final class IsGameOverTest: XCTestCase {
    func testHorizontal() {
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()
        board[0, 0] = Token.red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 6] = Token.yellow
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 1] = Token.red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 5] = Token.yellow
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 2] = Token.red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 4] = Token.yellow
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 3] = Token.red
        XCTAssertTrue(rules.isGameOver(board: board).result)
        XCTAssertEqual(rules.isGameOver(board: board).winner, Token.red)
    }
    func testFullGridWithoutWin() {
        // A REVOIR
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()

        // Remplir une grille complète sans qu'il y ait de victoire
        for row in 0..<board.rowNb {
            for col in 0..<board.columnNb {
                board[row, col] = (row + col) % 2 == 0 ? Token.red : Token.yellow
            }
        }

        // Personne ne gagne, mais la partie est terminée
        XCTAssertTrue(rules.isGameOver(board: board).result)
        XCTAssertEqual(rules.isGameOver(board: board).winner, Token.empty)
    }
    func testVertical() {
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()

        // Alternance des joueurs dans une colonne
        board[0, 0] = Token.red
        board[0, 1] = Token.yellow
        board[1, 0] = Token.red
        board[1, 1] = Token.yellow
        board[2, 0] = Token.red
        board[2, 1] = Token.yellow

        // Rouge complète la colonne
        board[3, 0] = Token.red

        // Le rouge gagne
        XCTAssertTrue(rules.isGameOver(board: board).result)
        XCTAssertEqual(rules.isGameOver(board: board).winner, Token.red)
    }
    func testDiagonalAscending() {
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()

        // Placer les jetons pour une victoire diagonale ascendante
        board[0, 0] = Token.red
        board[1, 1] = Token.yellow
        board[1, 0] = Token.red
        board[2, 2] = Token.yellow
        board[2, 1] = Token.red
        board[3, 3] = Token.yellow
        board[3, 2] = Token.red
        board[3, 1] = Token.yellow
        board[3, 0] = Token.red

        // Rouge gagne ici
        XCTAssertTrue(rules.isGameOver(board: board).result)
        XCTAssertEqual(rules.isGameOver(board: board).winner, Token.red)
    }

    func testDiagonalDescending() {
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()

        // Placer les jetons pour une victoire diagonale descendante
        board[3, 0] = Token.red
        board[2, 1] = Token.yellow
        board[2, 0] = Token.red
        board[1, 2] = Token.yellow
        board[1, 1] = Token.red
        board[0, 3] = Token.yellow
        board[0, 2] = Token.red
        board[0, 1] = Token.yellow
        board[0, 0] = Token.red

        // Rouge gagne ici
        XCTAssertTrue(rules.isGameOver(board: board).result)
        XCTAssertEqual(rules.isGameOver(board: board).winner, Token.red)
    }
}
